/*
 *  Copyright 2013-2050 Emmanuel BRUN (contact@amapj.fr)
 * 
 *  This file is part of AmapJ.
 *  
 *  AmapJ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  AmapJ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with AmapJ.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */
 package fr.amapj.view.views.saisiecontrat;

import com.vaadin.shared.ui.label.ContentMode;

import fr.amapj.common.AmapjRuntimeException;
import fr.amapj.model.models.contrat.modele.GestionPaiement;
import fr.amapj.service.services.gestioncontrat.reglesaisie.VerifRegleSaisieModeleContratDTO;
import fr.amapj.service.services.mescontrats.ContratDTO;
import fr.amapj.service.services.mescontrats.MesContratsService;
import fr.amapj.service.services.mescontrats.MonContratDTO;
import fr.amapj.service.services.mescontrats.inscription.InscriptionDTO;
import fr.amapj.service.services.stockservice.verifstock.VerifStockDTO;
import fr.amapj.view.engine.popup.PopupListener;
import fr.amapj.view.engine.popup.cascadingpopup.CInfo;
import fr.amapj.view.engine.popup.cascadingpopup.CascadingData;
import fr.amapj.view.engine.popup.cascadingpopup.CascadingPopup;
import fr.amapj.view.engine.popup.corepopup.CorePopup;
import fr.amapj.view.engine.popup.corepopup.CorePopup.ColorStyle;
import fr.amapj.view.engine.popup.formpopup.OnSaveException;
import fr.amapj.view.engine.popup.messagepopup.MessagePopup;
import fr.amapj.view.views.saisiecontrat.abo.joker.SaisieJoker;
import fr.amapj.view.views.saisiecontrat.table.PopupSaisieQteTable;
import fr.amapj.view.views.saisiecontrat.vueliste.PopupSaisieQteVueListe;

public class SaisieContrat
{
	private SaisieContratData data;
	private CascadingPopup cascading;
	
	public SaisieContrat(SaisieContratData data,PopupListener listener)
	{
		super();
		this.data = data;
		cascading = new CascadingPopup(listener,data);
	}

	public void doSaisie()
	{
		// On verifie d'abord si tout est OK 
		String msg = checkInitialCondition();
		if (msg!=null)
		{
			displayError(msg);
			return;
		}	
		
		// Si cheque seul : on passe directement à la suite
		if (data.modeSaisie==ModeSaisie.CHEQUE_SEUL)
		{
			cascading.start(getPopupPaiement());
			return;
		}
	
		// Si joker seul
		if (data.modeSaisie==ModeSaisie.JOKER)
		{
			SaisieJoker.doSaisieJokerOnly(data);	
			return;
		}
		
		// On execute le scenario de saisie
		CorePopup popup = null;
		switch (data.inscriptionDTO.popupType)
		{
		case POPUP_ABO:
			popup = new PopupSaisieQteContratPanier(data); 
			break;
			
		case POPUP_LIBRE:
		{
			popup = new PopupSaisieQteContrat(data);
			break;
			
			/* TODO 
			if (data.contratDTO.contratLigs.size()==1) // TODO
			{
				popup = new PopupSaisieQteTable(data); // TODO 
			}
			else if (data.contratDTO.contratLigs.size()>1) // TODO
			{
				popup = new PopupSaisieQteVueListe(data);  
			}
			else
			{
				popup = new PopupSaisieQteContrat(data);
			}
			break;*/
		}

		default:
			throw new AmapjRuntimeException();
		}
				
		CInfo info = new CInfo(popup,()->endOfSaisieQte());
		cascading.start(info);
	}
	


	private void displayError(String msg) 
	{
		MessagePopup p = new MessagePopup("Impossible de continuer",ContentMode.HTML,ColorStyle.RED,msg);
		CInfo info = new CInfo(p,null);
		cascading.start(info);
	}

	/**
	 * 
	 */
	private String checkInitialCondition()
	{
		//
		if (data.inscriptionDTO.errorMessage!=null)
		{
			return data.inscriptionDTO.errorMessage;
		}
		
		//
		switch (data.modeSaisie) 
		{
		case STANDARD:
			return checkInitialConditionSaisieStandard();

		// Dans les autres cas : aucune verif 
		default:
			return null;

		}
	}

	
	private String checkInitialConditionSaisieStandard()
	{	
		// On vérifie si le contrat est bien accessible par l'amapien   
		String msg = new MesContratsService().checkIfAccessAllowed(data.contratDTO,data.userId);
		return msg;
	}


	

	

	/**
	 * On a fini de saisir les quantités
	 * @return
	 */
	private CInfo endOfSaisieQte()
	{
		// Si que saisie des quantités : on sauvegarde tout de suite et on arrete 
		if (data.modeSaisie==ModeSaisie.QTE_SEUL)
		{
			try
			{
				data.saveContrat();
			} 
			catch (OnSaveException e)
			{
				e.showInNewDialogBox();
			}
			return null;
		}
		
		 // Sinon on continue avec le popup de paiement 
		return getPopupPaiement();
	
	}


	/**
	 * Retourne le popup à utiliser pour le paiement 
	 * @return
	 */
	protected CInfo getPopupPaiement()
	{
		if (data.contratDTO.paiement.gestionPaiement==GestionPaiement.GESTION_STANDARD)
		{
			return new CInfo(new PopupSaisiePaiement(data));
		}
		else
		{
			return new CInfo(new PopupInfoPaiement(data));
		}
	}
	
	
	
	

	/**
	 * Permet de lancer le cyle de saisie d'un contrat, avec les quantités et les reglements
	 * 
	 * @param contratDTO
	 * @param readOnly
	 * @param forTest
	 * @param userId
	 * @param messageSpecifique
	 */
	static public void saisieContrat(Long idModeleContrat,Long idContrat, Long userId, String messageSpecifique,ModeSaisie modeSaisie,PopupListener listener)
	{
		// Rechargement du contrat 
		MonContratDTO m = new MesContratsService().loadMonContratDTO(idModeleContrat,idContrat,modeSaisie);
				
		// Lancement de la saisie 
		SaisieContratData data = new SaisieContratData(m.contratDTO, m.inscriptionDTO,m.verifStockDTO,m.verifRegleSaisieDTO,userId, messageSpecifique,modeSaisie);
		SaisieContrat saisieContrat = new SaisieContrat(data, listener);
		saisieContrat.doSaisie();
	}
	
	public enum ModeSaisie
	{
		// Saisie standard faite par l'utilisateur
		STANDARD,
		
		// Saisie des jokers faite par l'utilisateur
		JOKER,
		
		// Mode test, correspond à la saisie standard faite par l'utilisateur 
		FOR_TEST,
		
		// Visualisation des informations du contrat 
		READ_ONLY,
		
		// Saisie de la quantité seule, pour correction par le referent
		QTE_SEUL,
		
		// Saisie des chéques seuls, pour correction par le referent
		CHEQUE_SEUL ,
		
		// Saisie de la quantité et des cheques , pour le referent qui saisit à la main le contrat des amapiens
		QTE_CHEQUE_REFERENT
		
	}
	
	
	public static class SaisieContratData extends CascadingData
	{
		public ContratDTO contratDTO;
		public InscriptionDTO inscriptionDTO;
		public VerifStockDTO verifStockDTO;
		public Long userId;
		public String messageSpecifique;
		public ModeSaisie modeSaisie;
		public VerifRegleSaisieModeleContratDTO verifRegleSaisieDTO;
				
		public SaisieContratData(ContratDTO contratDTO, InscriptionDTO inscriptionDTO,VerifStockDTO verifStockDTO, VerifRegleSaisieModeleContratDTO verifRegleSaisieDTO, Long userId, String messageSpecifique,ModeSaisie modeSaisie)
		{
			
			this.contratDTO = contratDTO;
			this.inscriptionDTO = inscriptionDTO;
			this.verifStockDTO = verifStockDTO;
			this.verifRegleSaisieDTO = verifRegleSaisieDTO;
			this.userId = userId;
			this.messageSpecifique = messageSpecifique;
			this.modeSaisie = modeSaisie;
		}
		
		
		/**
		 * Sauvegarde d'un contrat en fin de saisie, jette une exception si il y a des erreurs  
		 */
		public void saveContrat() throws OnSaveException
		{
			if (modeSaisie==ModeSaisie.FOR_TEST)
			{
				return;
			}
			new MesContratsService().saveNewContrat(contratDTO,userId);
		}

		
	}

}
